﻿using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace PACNET
{
    /// <summary>Memory Access API</summary>
    public partial class Memory
    {
        /// <summary>SRAM</summary>
        public const int PAC_MEM_SRAM = 0;

        /// <summary>EEPROM</summary>
        public const int PAC_MEM_EEPROM = 1;

        /// <summary>NVRAM</summary>
        public const int PAC_MEM_NVRAM = 2;

        //4.1. PAC_API DWORD pac_GetMemorySize(int mem_type)
        /// <summary>
        /// This function retrieves the size of the specified memory.
        /// </summary>
        /// <param name="mem_type">[in] Handle to a currently type memory.
        /// 0: pac_MEM_SRAM, 1: pac_MEM_EEPROM</param>
        /// <returns>The return value specifies the memory size.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetMemorySize")]
        public static extern uint GetMemorySize(int mem_type);

        //4.2. PAC_API bool pac_ReadMemory(DWORD address, LPBYTE lpBuffer, DWORD dwLength, int mem_type)
        /// <summary>
        /// This function retrieves the size of the specified memory.
        /// </summary>
        /// <param name="address">[in] Specifies the memory address where read from.</param>
        /// <param name="lpBuffer">[in] A pointer to a buffer that receives the memory data.</param>
        /// <param name="dwLength">[in] Number of characters to be read.</param>
        /// <param name="mem_type">[in] Handle to a currently type memory.
        /// 0: pac_MEM_SRAM, 1: pac_MEM_EEPROM</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_ReadMemory")]
        [return: MarshalAs(UnmanagedType.I1)]
        unsafe public static extern bool ReadMemory(uint address, byte* lpBuffer, uint dwLength, int mem_type);

        /// <summary>
        /// This function retrieves the size of the specified memory.
        /// </summary>
        /// <param name="index">[in] Specifies the memory address where read from.</param>
        /// <param name="Buffer">[in] A pointer to a buffer that receives the memory data.</param>
        /// <param name="Length">[in] Number of characters to be read.</param>
        /// <param name="mem_type">[in] Handle to a currently type memory.</param>
        unsafe public static void ReadMemory(uint index, byte[] Buffer, uint Length, int mem_type)
        {
            fixed (byte* pBuffer = Buffer)
                ReadMemory(index, pBuffer, Length, mem_type);
        }

        //4.3. PAC_API bool pac_WriteMemory(DWORD address, LPBYTE lpBuffer, DWORD dwLength, int mem_type)
        /// <summary>
        /// This function retrieves the size of the specified memory.
        /// </summary>
        /// <param name="address">[in] Specifies the memory address where write from.</param>
        /// <param name="lpBuffer">[in] A pointer to the buffer containing the data to be written to the memory.</param>
        /// <param name="dwLength">[in] Number of characters to be written.</param>
        /// <param name="mem_type">[in] Handle to a currently type memory.
        /// 0: pac_MEM_SRAM, 1: pac_MEM_EEPROM</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_WriteMemory")]
        [return: MarshalAs(UnmanagedType.I1)]
        unsafe public static extern bool WriteMemory(uint address, byte* lpBuffer, uint dwLength, int mem_type);

        /// <summary>
        /// This function retrieves the size of the specified memory.
        /// </summary>
        /// <param name="address">[in] Specifies the memory address where write from.</param>
        /// <param name="lpBuffer">[in] A pointer to the buffer containing the data to be written to the memory.</param>
        /// <param name="dwLength">[in] Number of characters to be written.</param>
        /// <param name="mem_type">[in] Handle to a currently type memory.
        /// 0: pac_MEM_SRAM, 1: pac_MEM_EEPROM</param>
        unsafe public static void WriteMemory(uint address, byte[] lpBuffer, uint dwLength, int mem_type)
        {
            fixed (byte* pBuffer = lpBuffer)
                WriteMemory(address, pBuffer, dwLength, mem_type);
        }

        //4.4. PAC_API void pac_EnableEEPROM(bool bEnable)
        /// <summary>
        /// This function turns on/off EEPROM.
        /// </summary>
        /// <param name="bEnable">[in] To turn on or protect EEPROM.
        /// True: To turn on EEPROM, False: To protect EEPROM.</param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_EnableEEPROM")]
        public static extern void EnableEEPROM(bool bEnable);
    }
}