﻿using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace PACNET
{
    /// <summary>Registry API</summary>
    public class PAC_Reg
    {
        /// <summary>NONE</summary>
        public const int PKT_NONE = 0;

        /// <summary>KEY</summary>
        public const int PKT_KEY = 1;

        /// <summary>STRING</summary>
        public const int PKT_STRING = 2;

        /// <summary>DWORD</summary>
        public const int PKT_DWORD = 3;


        //7.1. PAC_API DWORD pac_RegCountKey(LPCTSTR KeyName)
        /// <summary>
        /// This function retrieves the specified registry key which has how many sub keys.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key which you want to count.</param>
        /// <returns>Return the number of subkeys contained by the specified key.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegCountKey")]
        public static extern uint RegCountKey(string KeyName);

        //7.2. PAC_API DWORD pac_RegCountValue(LPCTSTR KeyName)
        /// <summary>
        /// This function retrieves the specified registry key which has how many values.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key which you want to count.</param>
        /// <returns>Return the number of values associated with the key.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegCountValue")]
        public static extern uint RegCountValue(string KeyName);

        //7.3. PAC_API bool pac_RegCreateKey(LPCTSTR KeyName)
        /// <summary>
        /// This function creates the specified registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key which you want to create.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegCreateKey")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegCreateKey(string KeyName);

        //7.4. PAC_API bool pac_RegDeleteKey(LPCTSTR KeyName)
        /// <summary>
        /// This function deletes a named subkey from the specified registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specifics the path of registry key which you want to delete.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegDeleteKey")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegDeleteKey(string KeyName);

        //7.5. PAC_API bool pac_RegDeleteValue(LPCTSTR KeyName)
        /// <summary>
        /// This function removes a named value from the specified registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specifics the path of registry key which you want to delete.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegDeleteValue")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegDeleteValue(string KeyName);

        //7.6. PAC_API DWORD pac_RegGetDWORD(LPCTSTR KeyName)
        /// <summary>
        /// This function retrieves value of the specified registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key.</param>
        /// <returns>Return the value of the specific key.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegGetDWORD")]
        public static extern uint RegGetDWORD(string KeyName);

        //7.7. PAC_API bool pac_RegGetKeyByIndex(LPCTSTR KeyName, DWORD dwIndex, LPTSTR lpName)
        /// <summary>
        /// This function retrieves the name of specified index of registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key.</param>
        /// <param name="dwIndex">[in] Specific the index of registry key.</param>
        /// <param name="lpName">[out] Assign a buffer to retrieves the specific the key name.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegGetKeyByIndex")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegGetKeyByIndex(string KeyName, uint dwIndex, string lpName);

        //7.8. PAC_API DWORD pac_RegGetKeyInfo(LPCTSTR KeyName)
        /// <summary>
        /// This function retrieves the type of specified index of registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key.</param>
        /// <returns>We define four types about the return value:
        /// 0: PKT_NONE, 1: PKT_KEY, 2: PKT_STRING, 3: PKT_DWORD</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegGetKeyInfo")]
        public static extern uint RegGetKeyInfo(string KeyName);

        //7.9. PAC_API bool pac_RegGetString(LPCTSTR KeyName, LPTSTR lpData, DWORD dwLength)
        /// <summary>
        /// This function retrieves value of the specified registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key.</param>
        /// <param name="lpData">[out] Pointer to a buffer that receives the value's data.</param>
        /// <param name="dwLength">[in] Specific the size of data.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegGetString")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegGetString(string KeyName, string lpData, uint dwLength);

        //7.10. PAC_API bool pac_RegGetValueByIndex(LPCTSTR KeyName, DWORD dwIndex, LPTSTR lpName)
        /// <summary>
        /// This function retrieves the value of specified index of registry key.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key.</param>
        /// <param name="dwIndex">[in] Specific the index of value.</param>
        /// <param name="lpName">[out] Pointer to a buffer that receives the value's data.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegGetValueByIndex")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegGetValueByIndex(string KeyName, uint dwIndex, string lpName);

        //7.11. PAC_API bool pac_RegKeyExist(LPCTSTR KeyName)
        /// <summary>
        /// This function determinants the specified registry key exist or not.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key which you want to check whether it exists or not.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegKeyExist")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegKeyExist(string KeyName);

        //7.12. PAC_API bool pac_RegSave(LPCTSTR KeyName)
        /// <summary>
        /// This function writes all the attributes of the specified open registry key into the registry.
        /// </summary>
        /// <param name="KeyName">[in] Handle to a currently open key or one of the following predefined reserved handle
        /// values:</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegSave")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegSave(string KeyName);

        //7.13. PAC_API bool pac_RegSetString(LPCTSTR KeyName, LPCTSTR assignStr, DWORD dwLength)
        /// <summary>
        /// This function assigns the specified registry key data whose type is string.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key which you want to assign data.</param>
        /// <param name="assignStr">[in] Specific the data.</param>
        /// <param name="dwLength">[in] Specific the size of data.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegSetString")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegSetString(string KeyName, string assignStr, uint dwLength);

        //7.14. PAC_API bool pac_RegSetDWORD(LPCTSTR KeyName, DWORD assignVal)
        /// <summary>
        /// This function assigns the specified registry key data whose type is DWORD.
        /// </summary>
        /// <param name="KeyName">[in] Specific the path of registry key which you want to assign data.</param>
        /// <param name="assignStr">[in] Specific the data.</param>
        /// <returns>Return true if success, otherwise false.</returns>    
        [DllImport("PACSDK.dll", EntryPoint = "pac_RegSetDWORD")]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool RegSetDWORD(string KeyName, uint assignStr);
    }
}